part of LanguageTools;

// Object describing some commun lexer methods
class LexerTool {
    String input;
    List<Token> output;
    int pos;
    Position position;
    Map<String, dynamic> META = new Map<String, dynamic>();

    // Initializing methods
    void setInput(String input) {
        this.input = input.replaceAll(new RegExp(r'\r\n'), '\n');
        output = new List<Token>();
        pos = 0;
        position = new Position();
        onSet();
    }

    // Following
    Position getPos() => position.clone();

    int charCode() => input.codeUnitAt(pos);

    void colAdvance([int number = 1]) {
        pos += number;
        position.col += number;
    }

    void rowAdvance([int number = 1]) {
        pos += number;
        position.row += number;
        position.col = 0;
    }

    Token addToken(Token token) {
        output.add(token);
        return token;
    }

    bool hasNextChar() => pos < input.length;

    Token get last => output.isEmpty ? null : output.last;

    // Lexing
    List<Token> tokenize() {
        Token token = new Token('');
        while(token is Token && !token.isNull) {
            token = nextToken();
        }
        return output;
    }

    // user defined
    void onSet() {

    }

    Token nextToken() {
        // must be imlemented by every Lexer
    }

    // Lexing
    String matchA(Symbol name, [int pos = null]) {
        if(pos == null) {
            pos = this.pos;
        }
        return REGEX[name].firstMatch(input.substring(pos))[0];
    }

    // Scanning methods
    bool isA(Symbol name, [int pos = null]) {
        if(pos == null) {
            pos = this.pos;
        }
        return REGEX[name].hasMatch(input.substring(pos));
    }

    // Constants & Meta
    Map<Symbol, RegExp> REGEX = new Map<Symbol, RegExp>();
}